﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

using SlidingWindowRange = System.Tuple<int, int>;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("SlidingWindowBestSolutionsCollection", "An object holding a collection of the best sliding window solutions.")]
  public abstract class SlidingWindowBestSolutionsCollection : Item {
    [Storable]
    private Dictionary<SlidingWindowRange, ISymbolicExpressionTree> bestSolutions;
    public Dictionary<SlidingWindowRange, ISymbolicExpressionTree> BestSolutions {
      get { return bestSolutions; }
      set { bestSolutions = value; }
    }

    [Storable]
    private IDataAnalysisProblemData problemData;
    public IDataAnalysisProblemData ProblemData {
      get { return problemData; }
      set { problemData = value; }
    }

    [Storable]
    private ISymbolicDataAnalysisExpressionTreeInterpreter interpreter;
    public ISymbolicDataAnalysisExpressionTreeInterpreter Interpreter {
      get { return interpreter; }
      set { interpreter = value; }
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    [StorableConstructor]
    protected SlidingWindowBestSolutionsCollection(bool deserializing) : base(deserializing) { }
    protected SlidingWindowBestSolutionsCollection(SlidingWindowBestSolutionsCollection original, Cloner cloner)
      : base(original, cloner) {
      this.bestSolutions = original.bestSolutions;
      this.problemData = original.problemData;
      this.interpreter = original.interpreter;
    }
    protected SlidingWindowBestSolutionsCollection() {
      bestSolutions = new Dictionary<SlidingWindowRange, ISymbolicExpressionTree>();
    }

    public bool ContainsKey(SlidingWindowRange key) {
      return bestSolutions.ContainsKey(key);
    }

    public ISymbolicExpressionTree this[SlidingWindowRange key] {
      get {
        return bestSolutions[key];
      }
      set {
        if (bestSolutions.ContainsKey(key))
          bestSolutions[key] = value;
        else
          bestSolutions.Add(key, value);
      }
    }

    public void Add(SlidingWindowRange range, ISymbolicExpressionTree solution) {
      bestSolutions.Add(range, solution);
    }

    public void Clear() {
      if (bestSolutions != null) bestSolutions.Clear();
    }

    public abstract ISymbolicDataAnalysisModel CreateModel(ISymbolicExpressionTree tree, ISymbolicDataAnalysisExpressionTreeInterpreter interpreter,
     double lowerEstimationLimit = double.MinValue, double upperEstimationLimit = double.MaxValue);

    public abstract ISymbolicDataAnalysisSolution CreateSolution(ISymbolicDataAnalysisModel model, IDataAnalysisProblemData problemData);
  }
}
